import os
from robot.api.deco import library, keyword
from robot.libraries.BuiltIn import BuiltIn
from robot.libraries.Process import Process

@library
class Commander:

    @property
    def Process(self) -> Process:
        self._process = self._process or self.BuiltIn.get_library_instance('Process')
        return self._process

    @property
    def BuiltIn(self) -> BuiltIn:
        self._builtin = self._builtin or BuiltIn()
        return self._builtin

    def __init__(self):
        self._process = None
        self._builtin = None
        self._operatingsystem = None

    @keyword
    def execute_pipeline(self, *commands, cwd=None, stdout=None, append=False):
        procs = []
        append = (stdout and append) or False
        procs.append(self.Process.start_process(
            *commands[0], cwd=cwd,
        ))
        for command in commands[1:-1]:
            procs.append(self.Process.start_process(
                *command, cwd=cwd,
                stdin=procs[-1].stdout,
            ))
        procs.append(self.Process.start_process(
            *commands[-1], cwd=cwd,
            stdin=procs[-1].stdout,
            stdout=None if append else stdout,
        ))
        if append:
            # set cwd like ProcessConfiguration does
            cwd = os.path.normpath(cwd) if cwd else os.path.abspath('.')
            # set output filename like ProcessConfiguration does
            stdout = os.path.normpath(os.path.join(cwd, stdout))
            procs.append(self.Process.start_process(
                'tee', '-a', stdout,
                cwd=cwd,
                stdin=procs[-1].stdout,
                stdout='DEVNULL',
            ))
        
        return self.Process.wait_for_process(procs[-1])
